from typing import List, Tuple

from .erreurs import ErreurArguments, TypeInconnu
from .chaines import chaine
from .entiers import entier 


def si(condition: bool, A, B=None, Aargs: list = [], Baargs: list = []):
    """
    Les fonctions entree dans cette fonction doivent avoir un argument qui est une liste si rien n est mis la liste sera vide
    Retourne le resultat de la fonction
    """
    if condition:
        return A(Aargs)
    else:
        return B(Baargs)


def dans(a : bool, L : List) -> bool:
    for i in L:
        if i == a:
            return True
    return False

def imprimer(*args) -> None:
    """Traduction de la fonction print"""
    res = chaine()
    for a in args:
        if isinstance(a, chaine):
            res += a
        else:
            res += chaine(a)
    print(res)

def pour(fonction, *args):
    """Equivalent de la fonction for + ajout du support de l ecriture c de for"""
    res = 0
    if len(args) > 3:
        raise ErreurArguments(
            "La fonction a été appelée avec trop d arguments")
    elif len(args) == 0:
        raise ErreurArguments("La fonction nécessite au moins 2 argument")
    elif len(args) == 1:
        if isinstance(args[0], (List, Tuple, str, chaine)):
            i = 0
            while i < len(args[0]):
                fonction(args[0][i])
                i += 1
        elif isinstance(args[0], (int, entier)):
            i = 0
            r = distance(args[0])
            while i < len(r):
                fonction(r[i])
                i += 1
    elif isinstance(args[0], (int, entier)):
        i = 0
        if len(args) == 2:
            r = distance(args[0], args[1])
        else:
            r = distance(args[0], args[1], args[2])
        while i < len(r):
            fonction(r[i])
            i += 1

def distance(*args) -> tuple:
    """Remplacement de la fonction range"""
    res = []
    if len(args) > 3:
        raise ErreurArguments(
            "La fonction a été appelée avec trop d arguments")
    elif len(args) == 0:
        raise ErreurArguments("La fonction nécessite au moins 1 argument")

    for v in args:
        if not isinstance(v, (int, entier)):
            raise TypeInconnu("Les arguments doivent etre des entiers")

    a = 0
    max = args[0]
    pas = 1

    if len(args) != 1:
        a = args[0]
        max = args[1]

    if len(args) == 3:
        pas = args[2]

    while a != max:
        res.append(a)
        a += pas
    return tuple(res)





from frpython.erreurs import TypeInconnu


class booleen:
    
    contenu : bool

    def __init__(self, o : object):
        
        

        if isinstance(o, (bool, booleen)):
            a = None
            if type(o) == bool:
                a = o
            else:
                a = o.contenu
            self.contenu = a

        elif isinstance(o, int):
            if o > 0:
                self.contenu = True
            else:
                self.contenu = False
        
        else:
            raise TypeInconnu("Le type entree ne peut etre converti en booleen")

    def __eq__(self, o: object) -> bool:
        if isinstance(o, bool):
            return self.contenu == o
        return self.contenu == o.contenu
    
    def __not__(self):
        if self.contenu:
            return booleen(False)
        return booleen(True)



from .entiers import entier


class chaine:
    """
    Pour les chaines de caracteres
    """
    contenu: str = ""
    taille: int = 0
    nombresStr = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]

    def __init__(self, o=""):
        res = []
        if isinstance(o, int):
            i = 10
            while o > i:
                res.append(o % i)
                i *= 10
            res.append(o % i)
            res.reverse()
            for n in res:
                self.contenu += self.nombresStr[n]
                self.taille += 1

        elif isinstance(o, str):
            self.contenu = o
            for _ in self.contenu:
                self.taille += 1

        elif isinstance(o, entier):
            i = 10
            while o.contenu > i:
                res.append(o.contenu % i)
                i *= 10
            res.append(o.contenu % i)
            res.reverse()
            for n in res:
                self.contenu += self.nombresStr[n]
                self.taille += 1

    def __add__(self, o: object):
        res = ""
        for c in self.contenu:
            res += c
        for c in o.contenu:
            res += c
        return chaine(res)

    def __str__(self) -> str:
        return self.contenu

    def __iter__(self):
        i = 0
        res = []
        while i < self.taille:
            res.append(self.contenu[i])
        return iter(res)




from .erreurs import ErreurDeConversion, TypeInconnu
from .decimaux import decimal
from .booleen import booleen

class entier:

    contenu: int = 0

    chiffres = (0, 1, 2, 3, 4, 5, 6, 7, 8, 9)
    dictioChiffres = {"0": 0, "1": 1, "2": 2, "3": 3,
                      "4": 4, "5": 5, "6": 6, "7": 7, "8": 8, "9": 9}

    def __init__(self, o : object) -> None:

        if isinstance(o, int):
            self.contenu = o

        elif isinstance(o, (str)):
            for c in o:
                if not (c in self.chiffres):
                    raise ErreurDeConversion(
                        f"Il faut uniquement des chiffres dans la chaine: {o}")
            l = []
            for chiffre in o[::-1]:
                self.contenu += self.dictioChiffres[chiffre]

        elif isinstance(o, (float, decimal)):
            if type(o) == float:
                a = str(o.contenu)
            else:
                a = str(o)
            o = a.split(".")[0]
            for chiffre in o[::-1]:
                self.contenu += self.dictioChiffres[chiffre]
            if int(a.split(".")[1][0]) >= 5:
                self.contenu += 1

        elif isinstance(o, (bool, booleen)):
            if o:
                self.contenu = 1
            else:
                self.contenu = 0

        else:
            raise TypeInconnu(
                "La variable entrée n'est pas convertible en entier")

    def vers_int(self):
        return self.contenu

    def __eq__(self, o: object) -> bool:
        if isinstance(o, entier):
            return self.contenu == o.contenu
        return self.contenu == o

    def __gt__(self, o: object) -> bool:
        if isinstance(o, entier):
            return self.contenu > o.contenu
        return self.contenu > o

    def __ge__(self, o: object) -> bool:
        if isinstance(o, entier):
            return self.contenu >= o.contenu
        return self.contenu >= o

    def __lt__(self, o: object) -> bool:
        if isinstance(o, entier):
            return self.contenu < o.contenu
        return self.contenu < o

    def __le__(self, o: object) -> bool:
        if isinstance(o, entier):
            return self.contenu <= o.contenu
        return self.contenu <= o

    def __truth__(self) -> bool:
        return bool(self.contenu)
    
    def __add__(self, o : object) -> object:
        if isinstance(o, int):
            return entier(self.contenu + o)
        return entier(self.contenu + o.contenu)

    def __sub__(self, o : object):
        if isinstance(o, int):
            return entier(self.contenu - o)
        return entier(self.contenu - o.contenu)

    def __truediv__(self, o : object) -> decimal:
        if isinstance(o, int):
            return decimal(self.contenu / o)
        return decimal(self.contenu / o.contenu)

    def __floordiv__(self, o : object) -> object:
        if isinstance(o, int):
            return entier(self.contenu // o)
        return entier(self.contenu // o.contenu)

    def __neg__(self):
        return entier(-self.contenu)

    def __pow__(self, o : object):
        if isinstance(o, int):
            return entier(self.contenu ** o)
        return entier(self.contenu ** o.contenu)

    def __mod__(self, o : object):
        if isinstance(o, int):
            return entier(self.contenu % o)
        return entier(self.contenu % o.contenu)

    def __mul__(self, o : object):
        if isinstance(o, int):
            return entier(self.contenu * o)
        return entier(self.contenu * o.contenu)

    


from typing import Any

from frpython.erreurs import TypeInconnu


class decimal:

    contenu: float

    def __init__(self, o: Any) -> None:
        if isinstance(o, float):
            self.contenu = float

    def __add__(self, o: object):
        if isinstance(o, float):
            return decimal(self.contenu + o)
        elif isinstance(o, decimal):
            return decimal(self.contenu + o.contenu)
        else:
            raise TypeInconnu(
                "Le type utilise ne peut etre additione a un decimal")

    def __eq__(self, o: object) -> bool:

        if isinstance(o, float):
            return self.contenu == o

        elif isinstance(o, decimal):
            return self.contenu == o.contenu

    def __add__(self, o : object):
        if isinstance(o, float):
            return decimal(self.contenu + o)
        return decimal(self.contenu + o.contenu)



from setuptools import find_packages, setup

setup(name="frpython", version='1.4', description='Tentative de traduction de python en francais',
      author="Gatien Oudoire", packages=find_packages(), url="https://github.com/Gatien-Oudoire/python-fr", download_url="https://pypi.org/project/frpython")



from .erreurs import *

from .decimaux import decimal
from .chaines import chaine
from typing import List, Tuple
from .entiers import entier
from .fonctions import *

def fr_python_info():
    try:
        imprimer("La librairie est prete")
    except:
        print("Erreur dans la librairie")


